#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""JSON
"""

# imports: library
from typing import Any, Generator


def indent(steps: int = 0, size: int = 2) -> str:
    """Indent"""

    return ' ' * size * steps


def formatter(unit: Any, indent_steps: int = 0) -> Generator:
    """Formatter"""

    if isinstance(unit, list):
        for index, item in enumerate(unit):

            if not isinstance(item, list) and not isinstance(item, dict):
                yield f'{indent(indent_steps)}[{index}] \'{item}\''
            else:
                yield f'{indent(indent_steps)}[{index}]'
                for subunit in formatter(item, indent_steps + 1):
                    yield subunit

    elif isinstance(unit, dict):
        for key, value in unit.items():
            start = f'{indent(indent_steps)}\'{key}\':'

            if not isinstance(value, list) and not isinstance(value, dict):
                yield f'{start} \'{value}\''
            else:
                yield start
                for item in formatter(value, indent_steps + 1):
                    yield item

    else:
        yield str(unit)



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Char str
"""

# imports: library
import string


def pseudo(value: int = ord('.')) -> str:
    """Pseudo"""

    return chr(value)


def _byte_to_printable_non_ws_or_space(value: int,
                                       default: str = '.',
                                       space: str = ' ',
                                       ) -> str:
    """Byte to printable non-WS or space"""

    char: str = chr(value)

    if char in ('\r', '\n'):
        return chr(0x21B5)  # ↵ Downwards Arrow with Corner Leftwards

    if char == ' ':
        return space

    if char in string.printable and char not in string.whitespace:
        return char

    return default


def byte_to_compact_printable_with_dots(value: int) -> str:
    """Byte to compact printable with dots"""

    default: str = '.'
    space: str = chr(0x22C5)  # ⋅ Dot Operator

    return _byte_to_printable_non_ws_or_space(value, default, space)


def byte_to_compact_printable_with_frames(value: int) -> str:
    """Byte to compact printable with frames"""

    default: str = chr(0x2395)  # ⎕ Apl Functional Symbol Quad
    space: str = chr(0x02FD)  # ⋅˽ Modifier Letter Shelf

    return _byte_to_printable_non_ws_or_space(value, default, space)



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Number str
"""


def pseudo(value: int,
           pad_to: int = 0,
           prefix: str = '',
           ) -> str:
    """Pseudo"""

    return prefix + str(value).zfill(pad_to)


def _number_str(name: str,
                base: str,
                value: int,
                pad_to: int = 0,
                prefix: str = None,
                ) -> str:
    """Number str"""

    if value < -1:
        raise ValueError(f'Negative value in number formatting: {value}')

    if value == -1:
        return name

    s_output = format(value, base)

    s_output = s_output.zfill(pad_to)

    if prefix is not None:
        if prefix in ('', base, f'0{base}'):
            s_output = f'0{base}' + s_output
        else:
            s_output = prefix + s_output

    return s_output


def hexadecimal(value: int,
                pad_to: int = 0,
                prefix: str = None,
                ) -> str:
    """Hexadecimal"""

    return hexadecimal_lower(value, pad_to, prefix)


def hexadecimal_lower(value: int,
                      pad_to: int = 0,
                      prefix: str = None,
                      ) -> str:
    """Hexadecimal - lower"""

    return _number_str('h', 'x', value, pad_to, prefix)


def hexadecimal_upper(value: int,
                      pad_to: int = 0,
                      prefix: str = None,
                      ) -> str:
    """Hexadecimal - upper"""

    return _number_str('h', 'X', value, pad_to, prefix)


def decimal(value: int,
            pad_to: int = 0,
            prefix: str = None
            ) -> str:
    """Decimal"""

    return _number_str('d', 'd', value, pad_to, prefix)


def octal(value: int,
          pad_to: int = 0,
          prefix: str = None
          ) -> str:
    """Octal"""

    return _number_str('o', 'o', value, pad_to, prefix)


def binary(value: int,
           pad_to: int = 0,
           prefix: str = None
           ) -> str:
    """Binary"""

    return _number_str('b', 'b', value, pad_to, prefix)



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Discord formatters
"""


def bold(text: str) -> str:
    """Bold"""

    return f'**{text}**'


def italic(text: str) -> str:
    """Italic"""

    return f'*{text}*'


def strikethrough(text: str) -> str:
    """Strike-through"""

    return f'~~{text}~~'


def quote(text: str) -> str:
    """Quote
    unofficial"""

    return f'"{text}"'


def code(text: str) -> str:
    """Code"""

    return f'`{text}`'


def hidden(text: str) -> str:
    """Hidden"""

    return f'||{text}||'



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Convert image from stream
"""

# imports: requirements
import PIL.Image

# modes: https://pillow.readthedocs.io/en/stable/handbook/concepts.html#concept-modes

# MODE "RGB" : (3x8-bit pixels, true color)

_DECODER_RAW: str = 'raw'


def rgb(folder: str,
        filename: str,
        data: bytes,
        size: tuple[int, int],
        scale: int = None) -> None:
    """RGB"""

    width, height = size

    img = PIL.Image.frombytes('RGB', size, data, decoder_name=_DECODER_RAW)
    img.save(f'{folder}/{filename}.png')

    if scale is not None:

        scaled_size = (width * 8, height * 8)

        img2 = PIL.Image.frombytes('RGB', size, data, decoder_name=_DECODER_RAW)
        img2 = img2.resize(scaled_size, resample=PIL.Image.NEAREST)
        img2.save(f'{folder}/{filename}-scale-{scale}.png')



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Terminal - argument types
"""

# imports: library
from argparse import ArgumentTypeError
import os.path


def type_file_path(path: str) -> str:
    """Argument parser file path type"""

    if not os.path.isfile(path):
        raise ArgumentTypeError(f'Argument "{path}" is not a valid path')

    return path



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Copy stubs
"""

# imports: library
import os


def copy_stubs(from_dir: str, to_dir: str, change: tuple[str, str]) -> None:
    """Copy stubs"""

    from_dir, to_dir = map(lambda x: x.strip(), (from_dir, to_dir))
    change_from, change_to = map(lambda x: x.strip(), change)

    if not os.path.exists(to_dir) or not os.path.isdir(to_dir):
        raise RuntimeError(f'TO dir does not exist: {to_dir}')

    try:
        file_list: list[str] = os.listdir(from_dir)
    except FileNotFoundError as exc:
        raise RuntimeError(f'FROM dir does not exist: {from_dir}') from exc
    except NotADirectoryError as exc:
        raise RuntimeError(f'FROM dir is not a directory: {from_dir}') from exc

    for filename_old in file_list:

        print('')
        print(' ' * 3, filename_old)

        filename, _ = os.path.splitext(filename_old)

        filename_new: str = filename.replace(change_from, change_to) + '.txt'

        print(' ->', filename_new)

        path_new: str = os.path.join(to_dir, filename_new)

        if os.path.isfile(path_new):
            print(' ', '#', 'Already existed.')
            continue

        with open(path_new, 'w', encoding='UTF-8') as fh_new:
            print('', file=fh_new)

        print(' ', '#', 'Done.')

    print('')


def main() -> None:
    """Main"""

    src_dir: str = input('Source dir: ')
    dest_dir: str = input('Destination dir: ')

    replace: str = input('Replace: ')
    replace_with: str = input('Replace with: ')

    try:
        copy_stubs(src_dir, dest_dir, (replace, replace_with))
    except RuntimeError as err:
        print(err)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Debugging
"""

# imports: project
from libmonty.outpututils.asciistyling import AsciiColor, AsciiStyle


def display_error_message(error: Exception) -> None:
    """Display error message"""

    print(AsciiStyle.bold((AsciiColor.red('Error')) + ', ' + error.args[0]))


def error_message(message: str) -> None:
    """Error message"""

    print(AsciiStyle.bold(AsciiColor.red('Error')) + ', ' + message)


def warning_message(message: str) -> None:
    """Warning message"""

    print(AsciiStyle.bold(AsciiColor.yellow('Warning')) + ', ' + message)


def info_message(message: str) -> None:
    """Info message"""

    print(AsciiStyle.bold(AsciiColor.blue('Info')) + ', ' + message)



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Directory
"""

# imports: library
import os
import re

# fields and initial values ####################################################

BASE_DIR: str = '~'
DATA_FILE_DICT_DELIMITER: str = '='


# operations ###################################################################

def join(path_1: str, path_2: str) -> str:
    """Join two directory paths"""

    return os.path.join(path_1, path_2)


# current directory information ################################################

def get_current_dir() -> str:
    """Get current dir"""

    return os.path.basename(os.getcwd())


def get_current_dir_path() -> str:
    """Get current dir path"""

    return os.getcwd()


def get_current_dir_content() -> dict:
    """Get current dir content"""

    content_list = os.listdir(os.getcwd())

    return {
        'list': content_list,
        'count': len(content_list)
    }


def get_current_dir_content_list() -> list[str]:
    """Get current dit content list"""

    return os.listdir(os.getcwd())


def get_current_dir_content_count() -> int:
    """Get current dir content count"""

    return len(os.listdir(os.getcwd()))


# checks #######################################################################

def is_dir(arg_path) -> bool:
    """Is dir?"""

    return os.path.isdir(arg_path)


def is_file(arg_path) -> bool:
    """Is file?"""

    return os.path.isfile(arg_path)


# navigation ###################################################################

def navigate(arg_path) -> None:
    """Navigate"""

    os.chdir(arg_path)


def nav_parent() -> None:
    """Navigate to parent"""

    os.chdir("../")


def nav_base_dir() -> None:
    """Navigate to base dir"""

    os.chdir(BASE_DIR)


# creation #####################################################################

def create_dir(name: str) -> None:
    """Create dir"""

    os.mkdir(name)


def create_file(name: str) -> None:
    """Create file"""

    with open(name, 'x', encoding='UTF-8') as _:
        pass


# special getters ##############################################################

def get_int_from_file(path: str) -> int:
    """Get int from file"""

    with open(path, 'r', encoding='UTF-8') as fh_value:
        return int(fh_value.readlines()[0].splitlines()[0])


def get_string_from_file(path: str) -> str:
    """Get string from file"""

    with open(path, 'r', encoding='UTF-8') as fh_value:
        return fh_value.readlines()[0].splitlines()[0]


def get_list_from_file(path: str) -> list[str]:
    """Get list from file"""

    with open(path, 'r', encoding='UTF-8') as fh_value:
        lines: list[str] = fh_value.readlines()

    return_list: list[str] = []

    for line in lines:
        line_content: str = line.splitlines()[0]

        non_data_content: re.Match = re.fullmatch('^[.*]?$', line_content)

        if not non_data_content and line_content != '':
            return_list.append(line.splitlines()[0].strip())

    return return_list


def get_dict_from_file(path: str) -> dict:
    """Get dict from file"""

    with open(path, "r", encoding='UTF-8') as fh_value:
        lines: list[str] = fh_value.readlines()

    return_dict: dict = {}

    for line in lines:
        line_content: str = line.splitlines()[0]

        non_data_content: re.Match = re.fullmatch('^[.*]?$', line_content)

        if not non_data_content and line_content != '':

            new_kv_pair: list[str] = line_content.split(DATA_FILE_DICT_DELIMITER)
            new_key: str = new_kv_pair[0].strip()
            new_value: str = new_kv_pair[1].strip()
            return_dict[new_key] = new_value

    return return_dict



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""HTTP responses
"""

# https://developer.mozilla.org/en-US/docs/Web/HTTP/Status
# https://en.wikipedia.org/wiki/List_of_HTTP_status_codes


def get(code: int) -> dict:
    """Get response code"""

    try:
        details: dict = _responses[code]
    except KeyError as exc:
        raise ValueError(f'Response code not found: {code}') from exc

    return details


_responses: dict[int, dict[str, str]] = {

    # ---------------------------------------------------------------- #
    # Informational responses (100–199)

    100: {
        'title': 'Continue',
        'description': (
            'This interim response indicates that everything so far is OK '
            'and that the client should continue the request, '
            'or ignore the response if the request is already finished.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/100'
    },
    101: {
        'title': 'Switching Protocol',
        'description': (
            'This code is sent in response to an Upgrade request header '
            'from the client, and indicates the protocol '
            'the server is switching to.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/101'
    },
    102: {
        'title': 'Processing (WebDAV)',
        'description': (
            'This code indicates that the server has received '
            'and is processing the request, '
            'but no response is available yet.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Glossary/WebDAV'
    },
    103: {
        'title': 'Early Hints',
        'description': (
            'This status code is primarily intended to be used '
            'with the Link header, letting the user agent start '
            'preloading resources while the server prepares a response.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/103'
    },

    # ---------------------------------------------------------------- #
    # Successful responses (200–299)

    200: {
        'title': 'OK',
        'description': (
            'The request has succeeded. '
            'The meaning of the success depends on the HTTP method: ' 
            'GET: The resource has been fetched '
            'and is transmitted in the message body. '
            'HEAD: The representation headers are included '
            'in the response without any message body. '
            'PUT or POST: The resource describing the result of the action '
            'is transmitted in the message body. '
            'TRACE: The message body contains the request message '
            'as received by the server.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/200'
    },
    201: {
        'title': 'Created',
        'description': (
            'The request has succeeded and '
            'a new resource has been created as a result. '
            'This is typically the response sent after POST requests, '
            'or some PUT requests.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/201'
    },
    202: {
        'title': 'Accepted',
        'description': (
            'The request has been received but not yet acted upon. '
            'It is noncommittal, since there is no way in HTTP to later send '
            'an asynchronous response indicating the outcome of the request. '
            'It is intended for cases where another process or server '
            'handles the request, or for batch processing.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/202'
    },
    203: {
        'title': 'Non-Authoritative Information',
        'description': (
            'This response code means the returned meta-information '
            'is not exactly the same as is available from the origin server, '
            'but is collected from a local or a third-party copy. '
            'This is mostly used for mirrors or backups of another resource. '
            'Except for that specific case, '
            'the \'200 OK\' response is preferred to this status.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/203'
    },
    204: {
        'title': 'No Content',
        'description': (
            'There is no content to send for this request, '
            'but the headers may be useful. '
            'The user-agent may update its cached headers '
            'for this resource with the new ones.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/204'
    },
    205: {
        'title': 'Reset Content',
        'description': (
            'Tells the user-agent '
            'to reset the document which sent this request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/205'
    },
    206: {
        'title': 'Partial Content',
        'description': (
            'This response code is used when the Range header is sent '
            'from the client to request only part of a resource.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/206'
    },
    207: {
        'title': 'Multi-Status (WebDAV)',
        'description': (
            'Conveys information about multiple resources, '
            'for situations where multiple status codes might be appropriate.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Glossary/WebDAV'
    },
    208: {
        'title': 'Already Reported (WebDAV)',
        'description': (
            'Used inside a <dav:propstat> response element '
            'to avoid repeatedly enumerating the internal members '
            'of multiple bindings to the same collection.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Glossary/WebDAV'
    },
    218: {
        'title': 'This is fine (Apache Web Server)',
        'description': (
            'Used as a catch-all error condition for allowing response bodies '
            'to flow through Apache when ProxyErrorOverride is enabled. '
            'When ProxyErrorOverride is enabled in Apache, '
            'response bodies that contain a status code of 4xx or 5xx '
            'are automatically discarded by Apache in favor of '
            'a generic response or a custom response specified '
            'by the ErrorDocument directive.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#218'
    },
    226: {
        'title': 'IM Used (HTTP Delta encoding)',
        'description': (
            'The server has fulfilled a GET request for the resource, '
            'and the response is a representation of the result '
            'of one or more instance-manipulations '
            'applied to the current instance.'
        ),
        'link': 'https://datatracker.ietf.org/doc/html/rfc3229'
    },

    # ---------------------------------------------------------------- #
    # Redirects (300–399)

    300: {
        'title': 'Multiple Choice',
        'description': (
            'The request has more than one possible response. '
            'The user-agent or user should choose one of them. '
            '(There is no standardized way of choosing one of the responses, '
            'but HTML links to the possibilities '
            'are recommended so the user can pick.)'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/300'
    },
    301: {
        'title': 'Moved Permanently',
        'description': (
            'The URL of the requested resource has been changed permanently. '
            'The new URL is given in the response.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/301'
    },
    302: {
        'title': 'Found',
        'description': (
            'This response code means that the URI of requested resource '
            'has been changed temporarily. '
            'Further changes in the URI might be made in the future. '
            'Therefore, this same URI should be used '
            'by the client in future requests.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/302'
    },
    303: {
        'title': 'See Other',
        'description': (
            'The server sent this response to direct the client '
            'to get the requested resource at another URI with a GET request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/303'
    },
    304: {
        'title': 'Not Modified',
        'description': (
            'This is used for caching purposes. '
            'It tells the client that the response has not been modified, '
            'so the client can continue to use '
            'the same cached version of the response.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/304'
    },
    305: {
        'title': 'Use Proxy',
        'description': (
            'Defined in a previous version of the HTTP specification to '
            'indicate that a requested response must be accessed by a proxy. '
            'It has been deprecated due to security concerns '
            'regarding in-band configuration of a proxy.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status'
    },
    306: {
        'title': 'unused',
        'description': (
            'This response code is no longer used; it is just reserved. '
            'It was used in a previous version of the HTTP/1.1 specification.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status'
    },
    307: {
        'title': 'Temporary Redirect',
        'description': (
            'The server sends this response to direct the client '
            'to get the requested resource at another URI '
            'with same method that was used in the prior request. '
            'This has the same semantics as the 302 Found HTTP response code, '
            'with the exception that the user agent '
            'must not change the HTTP method used: '
            'If a POST was used in the first request, '
            'a POST must be used in the second request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/307'
    },
    308: {
        'title': 'Permanent Redirect',
        'description': (
            'This means that the resource is now permanently located '
            'at another URI, specified by the Location: HTTP Response header. '
            'This has the same semantics as '
            'the 301 Moved Permanently HTTP response code, '
            'with the exception that the user agent must not change '
            'the HTTP method used: '
            'If a POST was used in the first request, '
            'a POST must be used in the second request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/308'
    },

    # ---------------------------------------------------------------- #
    # Client errors (400–499)

    400: {
        'title': 'Bad Request',
        'description': (
            'The server could not understand the request '
            'due to invalid syntax.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/400'
    },
    401: {
        'title': 'Unauthorized',
        'description': (
            'Although the HTTP standard specifies \"unauthorized\", '
            'semantically this response means \"unauthenticated\". '
            'That is, the client must authenticate itself '
            'to get the requested response.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/401'
    },
    402: {
        'title': 'Payment Required',
        'description': (
            'This response code is reserved for future use. '
            'The initial aim for creating this code '
            'was using it for digital payment systems, '
            'however this status code is used very rarely '
            'and no standard convention exists.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/402'
    },
    403: {
        'title': 'Forbidden',
        'description': (
            'The client does not have access rights to the content; '
            'that is, it is unauthorized, '
            'so the server is refusing to give the requested resource. '
            'Unlike 401, the client\'s identity is known to the server.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/403'
    },
    404: {
        'title': 'Not Found',
        'description': (
            'The server can not find the requested resource. '
            'In the browser, this means the URL is not recognized. '
            'In an API, this can also mean that the endpoint is valid '
            'but the resource itself does not exist. '
            'Servers may also send this response instead of 403 '
            'to hide the existence of a resource from an unauthorized client. '
            'This response code is probably the most famous one '
            'due to its frequent occurrence on the web.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/404'
    },
    405: {
        'title': 'Method Not Allowed',
        'description': (
            'The request method is known by the server '
            'but has been disabled and cannot be used. '
            'For example, an API may forbid DELETE-ing a resource. '
            'The two mandatory methods, GET and HEAD, '
            'must never be disabled and should not return this error code.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/405'
    },
    406: {
        'title': 'Not Acceptable',
        'description': (
            'This response is sent when the web server, '
            'after performing server-driven content negotiation, '
            'doesn\'t find any content that conforms to '
            'the criteria given by the user agent.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/406'
    },
    407: {
        'title': 'Proxy Authentication Required',
        'description': (
            'This is similar to 401 '
            'but authentication is needed to be done by a proxy.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/407'
    },
    408: {
        'title': 'Request Timeout',
        'description': (
            'This response is sent on an idle connection by some servers, '
            'even without any previous request by the client. '
            'It means that the server would like to '
            'shut down this unused connection. '
            'This response is used much more since some browsers, '
            'like Chrome, Firefox 27+, or IE9, '
            'use HTTP pre-connection mechanisms to speed up surfing. '
            'Also note that some servers merely shut down the connection '
            'without sending this message.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/408'
    },
    409: {
        'title': 'Conflict',
        'description': (
            'This response is sent when a request conflicts '
            'with the current state of the server.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/409'
    },
    410: {
        'title': 'Gone',
        'description': (
            'This response is sent when the requested content '
            'has been permanently deleted from server, '
            'with no forwarding address. '
            'Clients are expected to remove their caches '
            'and links to the resource. '
            'The HTTP specification intends this status code '
            'to be used for \"limited-time, promotional services\". '
            'APIs should not feel compelled to indicate resources '
            'that have been deleted with this status code.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/410'
    },
    411: {
        'title': 'Length Required',
        'description': (
            'Server rejected the request because '
            'the Content-Length header field is not defined '
            'and the server requires it.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/411'
    },
    412: {
        'title': 'Precondition Failed',
        'description': (
            'The client has indicated preconditions in its headers '
            'which the server does not meet.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/412'
    },
    413: {
        'title': 'Payload Too Large',
        'description': (
            'Request entity is larger than limits defined by server; '
            'the server might close the connection '
            'or return an Retry-After header field.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/413'
    },
    414: {
        'title': 'URI Too Long',
        'description': (
            'The URI requested by the client is longer '
            'than the server is willing to interpret.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/414'
    },
    415: {
        'title': 'Unsupported Media Type',
        'description': (
            'The media format of the requested data is not supported by the server, '
            'so the server is rejecting the request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/415'
    },
    416: {
        'title': 'Range Not Satisfiable',
        'description': (
            'The range specified by the Range header field in the request '
            'can\'t be fulfilled; it\'s possible that the range '
            'is outside the size of the target URI\'s data.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/416'
    },
    417: {
        'title': 'Expectation Failed',
        'description': (
            'This response code means the expectation indicated '
            'by the Expect request header field can\'t be met by the server.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/417'
    },
    418: {
        'title': 'I\'m a teapot',
        'description': (
            'The server refuses the attempt '
            'to brew coffee with a teapot.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/418'
    },
    419: {
        'title': 'Page Expired (Laravel Framework)',
        'description': (
            'Used by the Laravel Framework '
            'when a CSRF Token is missing or expired.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#419'
    },
    420: {
        'title': 'Method Failure (Spring Framework)',
        'description': (
            'A deprecated response used by the Spring Framework '
            'when a method has failed.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#420'
    },
    421: {
        'title': 'Misdirected Request',
        'description': (
            'The request was directed at a server '
            'that is not able to produce a response. '
            'This can be sent by a server that is not configured '
            'to produce responses for the combination '
            'of scheme and authority that are included in the request URI.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status'
    },
    422: {
        'title': 'Unprocessable Entity (WebDAV)',
        'description': (
            'The request was well-formed '
            'but was unable to be followed due to semantic errors.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/422'
    },
    423: {
        'title': 'Locked (WebDAV)',
        'description': (
            'The resource that is being accessed '
            'is locked.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Glossary/WebDAV'
    },
    424: {
        'title': 'Failed Dependency (WebDAV)',
        'description': (
            'The request failed '
            'due to failure of a previous request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Glossary/WebDAV'
    },
    425: {
        'title': 'Too Early',
        'description': (
            'Indicates that the server is unwilling '
            'to risk processing a request that might be replayed.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/425'
    },
    426: {
        'title': 'Upgrade Required',
        'description': (
            'The server refuses to perform the request '
            'using the current protocol but might be willing to do so '
            'after the client upgrades to a different protocol. '
            'The server sends an Upgrade header in a 426 response '
            'to indicate the required protocol(s).'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/426'
    },
    428: {
        'title': 'Precondition Required',
        'description': (
            'The origin server requires the request to be conditional. '
            'This response is intended to prevent the \'lost update\' problem, '
            'where a client GETs a resource\'s state, '
            'modifies it, and PUTs it back to the server, '
            'when meanwhile a third party has modified the state on the server, '
            'leading to a conflict.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/428'
    },
    429: {
        'title': 'Too Many Requests',
        'description': (
            'The user has sent too many requests '
            'in a given amount of time (\"rate limiting\").'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/429'
    },
    430: {
        'title': 'Request Header Fields Too Large (Shopify)',
        'description': (
            'Used by Shopify, '
            'instead of the 429 Too Many Requests response code, '
            'when too many URLs are requested within a certain time frame.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes'
    },
    431: {
        'title': 'Request Header Fields Too Large',
        'description': (
            'The server is unwilling to process the request '
            'because its header fields are too large. '
            'The request may be resubmitted '
            'after reducing the size of the request header fields.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/431'
    },
    440: {
        'title': 'Login Time-out (Microsoft IIS)',
        'description': (
            'The client\'s session has expired '
            'and must log in again.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#440'
    },
    444: {
        'title': 'No Response (nginx)',
        'description': (
            'Used internally to instruct the server to return '
            'no information to the client and close the connection immediately.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#444'
    },
    449: {
        'title': 'Retry With (Microsoft IIS)',
        'description': (
            'The server cannot honour the request '
            'because the user has not provided the required information.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#449'
    },
    450: {
        'title': 'Blocked by Windows Parental Controls (Microsoft)',
        'description': (
            'The Microsoft extension code indicated '
            'when Windows Parental Controls are turned on and '
            'are blocking access to the requested webpage.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#450'
    },
    451: {
        'title': 'Unavailable For Legal Reasons',
        'description': (
            'The user-agent requested a resource '
            'that cannot legally be provided, '
            'such as a web page censored by a government.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/451'
    },
    460: {
        'title': '(AWS Elastic Load Balancer)',
        'description': (
            'Client closed the connection with the load balancer '
            'before the idle timeout period elapsed. '
            'Typically when client timeout is sooner '
            'than the Elastic Load Balancer\'s timeout.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#460'
    },
    463: {
        'title': '(AWS Elastic Load Balancer)',
        'description': (
            'The load balancer received an X-Forwarded-For request header '
            'with more than 30 IP addresses.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#463'
    },
    494: {
        'title': 'Request header too large (nginx)',
        'description': (
            'Client sent too large request '
            'or too long header line.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#494'
    },
    495: {
        'title': 'SSL Certificate Error (nginx)',
        'description': (
            'An expansion of the 400 Bad Request response code, '
            'used when the client has provided an invalid client certificate.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#495'
    },
    496: {
        'title': 'SSL Certificate Required (nginx)',
        'description': (
            'An expansion of the 400 Bad Request response code, '
            'used when a client certificate is required but not provided.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#496'
    },
    497: {
        'title': 'HTTP Request Sent to HTTPS Port (nginx)',
        'description': (
            'An expansion of the 400 Bad Request response code, '
            'used when the client has made a HTTP request to a port '
            'listening for HTTPS requests.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#497'
    },
    498: {
        'title': 'Invalid Token (Esri)',
        'description': (
            'Returned by ArcGIS for Server. '
            'Code 498 indicates an expired or otherwise invalid token.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#498'
    },
    499: {
        'title': 'Token Required (Esri) / Client Closed Request (nginx)',
        'description': (
            'Returned by ArcGIS for Server. '
            'Code 499 indicates that a token is required but was not submitted.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#499'
    },

    # ---------------------------------------------------------------- #
    # Server errors (500–599)

    500: {
        'title': 'Internal Server Error',
        'description': (
            'The server has encountered a situation '
            'it doesn\'t know how to handle.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/500'
    },
    501: {
        'title': 'Not Implemented',
        'description': (
            'The request method is not supported by the server '
            'and cannot be handled. '
            'The only methods that servers are required to support '
            '(and therefore that must not return this code) are GET and HEAD.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/501'
    },
    502: {
        'title': 'Bad Gateway',
        'description': (
            'This error response means that the server, '
            'while working as a gateway to get a response needed '
            'to handle the request, got an invalid response.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/502'
    },
    503: {
        'title': 'Service Unavailable',
        'description': (
            'The server is not ready to handle the request. '
            'Common causes are a server that is down for maintenance '
            'or that is overloaded. '
            'Note that together with this response, '
            'a user-friendly page explaining the problem should be sent. '
            'This responses should be used for temporary conditions '
            'and the Retry-After: HTTP header should, if possible, '
            'contain the estimated time before the recovery of the service. '
            'The webmaster must also take care about '
            'the caching-related headers '
            'that are sent along with this response, '
            'as these temporary condition responses s'
            'hould usually not be cached.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/503'
    },
    504: {
        'title': 'Gateway Timeout',
        'description': (
            'This error response is given when the server '
            'is acting as a gateway and cannot get a response in time.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/504'
    },
    505: {
        'title': 'HTTP Version Not Supported',
        'description': (
            'The HTTP version used in the request '
            'is not supported by the server.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/505'
    },
    506: {
        'title': 'Variant Also Negotiates',
        'description': (
            'The server has an internal configuration error: '
            'the chosen variant resource is configured '
            'to engage in transparent content negotiation itself, '
            'and is therefore not a proper end point '
            'in the negotiation process.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/506'
    },
    507: {
        'title': 'Insufficient Storage (WebDAV)',
        'description': (
            'The method could not be performed on the resource '
            'because the server is unable to store the representation needed '
            'to successfully complete the request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/507'
    },
    508: {
        'title': 'Loop Detected (WebDAV)',
        'description': (
            'The server detected an infinite loop '
            'while processing the request.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/508'
    },
    509: {
        'title': 'Bandwidth Limit Exceeded (Apache Web Server/cPanel)',
        'description': (
            'The server has exceeded the bandwidth '
            'specified by the server administrator; '
            'this is often used by shared hosting providers '
            'to limit the bandwidth of customers.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#509'
    },
    510: {
        'title': 'Not Extended',
        'description': (
            'Further extensions to the request are required '
            'for the server to fulfill it.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/510'
    },
    511: {
        'title': 'Network Authentication Required',
        'description': (
            'The 511 status code indicates '
            'that the client needs to authenticate to gain network access.'
        ),
        'link': 'https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/511'
    },
    520: {
        'title': 'Web Server Returned an Unknown Error (Cloudflare)',
        'description': (
            'The origin server returned an empty, unknown, '
            'or unexplained response to Cloudflare.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#520'
    },
    521: {
        'title': 'Web Server Is Down (Cloudflare)',
        'description': (
            'Error 521 occurs when the origin web server '
            'refuses connections from Cloudflare. '
            'Security solutions at your origin '
            'may block legitimate connections '
            'from certain Cloudflare IP addresses.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#521'
    },
    522: {
        'title': 'Connection Timed Out (Cloudflare)',
        'description': (
            'Error 522 occurs when Cloudflare times out '
            'contacting the origin web server.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#522'
    },
    523: {
        'title': 'Origin Is Unreachable (Cloudflare)',
        'description': (
            'Cloudflare could not reach the origin server; for example, '
            'if the DNS records for the origin server are incorrect.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#523'
    },
    524: {
        'title': 'A Timeout Occurred (Cloudflare)',
        'description': (
            'Cloudflare was able to complete a TCP connection '
            'to the origin server, '
            'but did not receive a timely HTTP response.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#524'
    },
    525: {
        'title': 'SSL Handshake Failed (Cloudflare)',
        'description': (
            'Cloudflare could not negotiate a SSL/TLS handshake '
            'with the origin server.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#525'
    },
    526: {
        'title': 'Invalid SSL Certificate (Cloudflare)',
        'description': (
            'Cloudflare could not validate the SSL certificate '
            'on the origin web server. '
            'Also used by Cloud Foundry\'s gorouter.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#526'
    },
    527: {
        'title': 'Railgun Error (Cloudflare)',
        'description': (
            'Error 527 indicates an interrupted connection between Cloudflare '
            'and the origin server\'s Railgun server.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#527'
    },
    529: {
        'title': 'Site is overloaded (Qualys)',
        'description': (
            'Used by Qualys in the SSLLabs server testing API '
            'to signal that the site can\'t process the request.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#529'
    },
    530: {
        'title': 'Site is frozen (Pantheon) / [along with a 1xxx] (Cloudflare)',
        'description': (
            'Used by the Pantheon web platform '
            'to indicate a site that has been frozen due to inactivity.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#530'
    },
    561: {
        'title': 'Unauthorized (AWS Elastic Load Balancer)',
        'description': (
            'An error around authentication returned by a server '
            'registered with a load balancer. '
            'You configured a listener rule to authenticate users, '
            'but the identity provider (IdP) returned an error code '
            'when authenticating the user.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#561'
    },
    598: {
        'title': '(Informal convention) Network read timeout error',
        'description': (
            'Used by some HTTP proxies to signal a network read timeout '
            'behind the proxy to a client in front of the proxy.'
        ),
        'link': 'https://en.wikipedia.org/wiki/List_of_HTTP_status_codes#598'
    }
}



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

""" Output styling with ASCII escape sequences
    Version 1.1

    https://en.wikipedia.org/wiki/ANSI_escape_code

    Python utilites
    https://github.com/sunarch/libmonty
    'utils' folder
"""


# style ########################################################################

class AsciiStyle:
    """ASCII style"""

    @staticmethod
    def bold(printable: str) -> str:
        """Bold"""

        return f'\x1b[1m{printable}\x1b[0m'

    @staticmethod
    def faint(printable: str) -> str:
        """Faint"""

        return f'\x1b[2m{printable}\x1b[0m'

    @staticmethod
    def italic(printable: str) -> str:
        """Italic"""

        return f'\x1b[3m{printable}\x1b[0m'

    @staticmethod
    def underlined(printable: str) -> str:
        """Underlined"""

        return f'\x1b[4m{printable}\x1b[0m'

    @staticmethod
    def blink(printable: str) -> str:
        """Blink"""

        return f'\x1b[5m{printable}\x1b[0m'

    @staticmethod
    def image_negative(printable: str) -> str:
        """Image negative"""

        return f'\x1b[7m{printable}\x1b[0m'

    @staticmethod
    def framed(printable: str) -> str:
        """Framed"""

        return f'\x1b[51m{printable}\x1b[0m'

    @staticmethod
    def encircled(printable: str) -> str:
        """Encircled"""

        return f'\x1b[52m{printable}\x1b[0m'

    @staticmethod
    def overlined(printable: str) -> str:
        """Overlined"""

        return f'\x1b[53m{printable}\x1b[0m'


# font #########################################################################

class AsciiFont:
    """ASCII font"""

    @staticmethod
    def primary(printable: str) -> str:
        """Primary"""

        return f'\x1b[10m{printable}\x1b[0m'

    @staticmethod
    def alternate(printable: str, arg_alternate_font_no) -> str:
        """Alternate"""

        inserted_no = '1'

        if int(arg_alternate_font_no) in range(1, 10):  # 0 < x < 10 #
            inserted_no = str(arg_alternate_font_no)

        return f'\x1b[1{inserted_no}m{printable}\x1b[0m'


# color ########################################################################

class AsciiColor:
    """ASCII color"""

    @staticmethod
    def black(printable: str) -> str:
        """Black"""

        return f'\x1b[30m{printable}\x1b[0m'

    @staticmethod
    def red(printable: str) -> str:
        """Red"""

        return f'\x1b[31m{printable}\x1b[0m'

    @staticmethod
    def green(printable: str) -> str:
        """Green"""

        return f'\x1b[32m{printable}\x1b[0m'

    @staticmethod
    def yellow(printable: str) -> str:
        """Yellow"""

        return f'\x1b[33m{printable}\x1b[0m'

    @staticmethod
    def blue(printable: str) -> str:
        """Blue"""

        return f'\x1b[34m{printable}\x1b[0m'

    @staticmethod
    def magenta(printable: str) -> str:
        """Magenta"""

        return f'\x1b[35m{printable}\x1b[0m'

    @staticmethod
    def cyan(printable: str) -> str:
        """Cyan"""

        return f'\x1b[36m{printable}\x1b[0m'

    @staticmethod
    def white(printable: str) -> str:
        """White"""

        return f'\x1b[37m{printable}\x1b[0m'


# background ###################################################################

class AsciiBackground:
    """ASCII background"""

    @staticmethod
    def black(printable: str) -> str:
        """Black"""

        return f'\x1b[40m{printable}\x1b[0m'

    @staticmethod
    def red(printable: str) -> str:
        """Red"""

        return f'\x1b[41m{printable}\x1b[0m'

    @staticmethod
    def green(printable: str) -> str:
        """Green"""

        return f'\x1b[42m{printable}\x1b[0m'

    @staticmethod
    def yellow(printable: str) -> str:
        """Yellow"""

        return f'\x1b[43m{printable}\x1b[0m'

    @staticmethod
    def blue(printable: str) -> str:
        """Blue"""

        return f'\x1b[44m{printable}\x1b[0m'

    @staticmethod
    def magenta(printable: str) -> str:
        """Magenta"""

        return f'\x1b[45m{printable}\x1b[0m'

    @staticmethod
    def cyan(printable: str) -> str:
        """Cyan"""

        return f'\x1b[46m{printable}\x1b[0m'

    @staticmethod
    def white(printable: str) -> str:
        """White"""

        return f'\x1b[47m{printable}\x1b[0m'



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Main"""

# imports: library
from argparse import ArgumentParser
import sys

# imports: dependencies
import libmonty_logging
import libmonty_logging.message as logging_message

# imports: project
from libmonty import version


def main() -> None:
    """Main"""

    libmonty_logging.apply_default_console_and_file(
        version.PROGRAM_NAME,
        version.__version__
    )

    logging_message.program_header(version.PROGRAM_NAME)

    parser = ArgumentParser(prog=version.PROGRAM_NAME)

    parser.add_argument('--version',
                        help='Display version',
                        action='store_true',
                        dest='version')

    args = parser.parse_args(sys.argv[1:])

    if args.version:
        print(f'{version.PROGRAM_NAME} {version.__version__}')
        return


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Wave PCM audio
"""

# import: requirements
from tqdm import tqdm


class WavePCMAudio:
    """Wave PCM audio"""

    DEFAULT_NUM_CHANNELS: int = 1
    DEFAULT_SAMPLE_RATE: int = 44100
    DEFAULT_BITS_PER_SAMPLE: int = 16

    # format source: http://soundfile.sapp.org/doc/WaveFormat/
    # The Canonical WAVE PCM Soundfile Format

    def __init__(self,
                 num_channels: int = DEFAULT_NUM_CHANNELS,
                 sample_rate: int = DEFAULT_SAMPLE_RATE,
                 bits_per_sample: int = DEFAULT_BITS_PER_SAMPLE) -> None:
        """Initialize"""

        print('Initializing WavePCMAudioClass object ...', end='')

        # init instance variables
        self._num_channels = self.DEFAULT_NUM_CHANNELS
        self._sample_rate = self.DEFAULT_SAMPLE_RATE
        self._bits_per_sample = self.DEFAULT_BITS_PER_SAMPLE
        self._data = bytearray()

        # init values from arguments
        self.num_channels = num_channels
        self.sample_rate = sample_rate
        self.bits_per_sample = bits_per_sample

        print(' done.')

        self.print_format_info()

    ############################################################################
    # The canonical WAVE format starts with the RIFF header: ###################

    # ChunkID ##################################################################

    # Offset: 0, Size: 4, Endian: big
    # Contains the letters 'RIFF' in ASCII form
    # (0x52494646 big-endian form)

    @property
    def chunk_id(self) -> str:
        """Chunk ID"""

        return 'RIFF'

    @property
    def chunk_id_bytes(self) -> bytes:
        """Chunk ID bytes"""

        return bytes(self.chunk_id, encoding='ASCII')

    # ChunkSize ################################################################

    # Offset: 4, Size: 4, Endian: little
    # 36 + SubChunk2Size, or more precisely:
    # 4 + (8 + SubChunk1Size) + (8 + SubChunk2Size)
    # This is the size of the rest of the chunk following this number.
    # This is the size of the entire file in bytes minus 8 bytes for the
    # two fields not included in this count: ChunkID and ChunkSize.

    @property
    def chunk_size(self) -> int:
        """Chunk size"""

        return 36 + self.subchunk_2_size

    @property
    def chunk_size_bytes(self) -> bytes:
        """Chunk size bytes"""

        return self.chunk_size.to_bytes(4, byteorder='little', signed=False)

    # Format ###################################################################

    # Offset: 8, Size: 4, Endian: big
    # Contains the letters 'WAVE'
    # (0x57415645 big-endian form)

    @property
    def format(self) -> str:
        """Format"""

        return 'WAVE'

    @property
    def format_bytes(self) -> bytes:
        """Format bytes"""

        return bytes(self.format, encoding='ASCII')

    ############################################################################
    # The 'WAVE' format consists of two subchunks: 'fmt ' and 'data': ##########

    ############################################################################
    # The 'fmt ' subchunk describes the sound data's format: ###################

    # Subchunk1ID ##############################################################

    # Offset: 12, Size: 4, Endian: big
    # Contains the letters 'fmt '
    # (0x666d7420 big-endian form)

    @property
    def subchunk_1_id(self) -> str:
        """Subchunk 1 ID"""

        return 'fmt '

    @property
    def subchunk_1_id_bytes(self) -> bytes:
        """Subchunk 1 ID bytes"""

        return bytes(self.subchunk_1_id, encoding='ASCII')

    # Subchunk1Size ############################################################

    # Offset: 16, Size: 4, Endian: little
    # 16 for PCM.
    # This is the size of the rest of the Subchunk which follows this number.

    @property
    def subchunk_1_size(self) -> int:
        """Subchunk 1 size"""

        return 16

    @property
    def subchunk_1_size_bytes(self) -> bytes:
        """Subchunk 1 size bytes"""

        return self.subchunk_1_size.to_bytes(4, byteorder='little', signed=False)

    # AudioFormat ##############################################################

    # Offset: 20, Size: 2, Endian: little
    # PCM = 1 (i.e. Linear quantization)
    # Values other than 1 indicate some form of compression.

    @property
    def audio_format(self) -> int:
        """Audio format"""

        return 1

    @property
    def audio_format_bytes(self) -> bytes:
        """Audio format bytes"""

        return self.audio_format.to_bytes(2, byteorder='little', signed=False)

    # NumChannels ##############################################################

    # Offset: 22, Size: 2, Endian: little
    # Mono = 1, Stereo = 2, etc.

    @property
    def num_channels(self) -> int:
        """Num channels"""

        return self._num_channels

    @num_channels.setter
    def num_channels(self, new_value: int) -> None:
        """Set num channels"""

        if new_value < 1:
            raise ValueError('Channel count can\'t be lower than 1')
        if new_value > 8:
            raise ValueError('Channel count can\'t be higher than 8')

        self._num_channels = new_value

    @property
    def num_channels_bytes(self) -> bytes:
        """Num channels - bytes"""

        return self.num_channels.to_bytes(2, byteorder='little', signed=False)

    # SampleRate ###############################################################

    # Offset: 24, Size: 4, Endian: little
    # 8000, 44100, etc.

    @property
    def sample_rate(self) -> int:
        """Sample rate"""

        return self._sample_rate

    @sample_rate.setter
    def sample_rate(self, new_value: int) -> None:
        """Set - sample rate"""

        if new_value < 8000:
            raise ValueError('Sample rate can\'t be lower than 8 KHz')
        if new_value > 96000:
            raise ValueError('Sample rate can\'t be higher than 96 KHz')

        self._sample_rate = new_value

    @property
    def sample_rate_bytes(self) -> bytes:
        """Sample rate bytes"""

        return self.sample_rate.to_bytes(4, byteorder='little', signed=False)

    # ByteRate #################################################################

    # Offset: 28, Size: 4, Endian: little
    # == SampleRate * NumChannels * BitsPerSample/8

    @property
    def byte_rate(self) -> int:
        """Byte rate"""

        return int(self.sample_rate * self.num_channels * self.bits_per_sample / 8)

    @property
    def byte_rate_bytes(self) -> bytes:
        """Byte rate bytes"""

        return self.byte_rate.to_bytes(4, byteorder='little', signed=False)

    # BlockAlign ###############################################################

    # Offset: 32, Size: 2, Endian: little
    # == NumChannels * BitsPerSample/8
    # The number of bytes for one sample including all channels.
    # I wonder what happens when this number isn't an integer?

    @property
    def block_align(self) -> int:
        """Block align"""

        return int(self.num_channels * self.bits_per_sample / 8)

    @property
    def block_align_bytes(self) -> bytes:
        """Block align bytes"""

        return self.block_align.to_bytes(2, byteorder='little', signed=False)

    # BitsPerSample ############################################################

    # Offset: 34, Size: 2, Endian: little
    # 8 bits = 8, 16 bits = 16, etc.

    @property
    def bits_per_sample(self) -> int:
        """Bits per sample"""

        return self._bits_per_sample

    @bits_per_sample.setter
    def bits_per_sample(self, new_value: int) -> None:
        """Bits per sample"""

        if new_value in {8, 16, 24, 32}:
            self._bits_per_sample = new_value
        else:
            raise ValueError('Valid values for Bits per Sample: 8, 16, 24 or 32')

    @property
    def bits_per_sample_bytes(self) -> bytes:
        """Bits per sample bytes"""

        return self.bits_per_sample.to_bytes(2, byteorder='little', signed=False)

    ######################################

    @property
    def is_sample_format_signed(self) -> bool:
        """Is sample format signed?"""

        return False

    @property
    def sample_format_signed(self) -> str:
        """Sample format signed"""

        return 'S' if self.is_sample_format_signed else 'U'

    @property
    def is_sample_format_big_endian(self) -> bool:
        """Is sample format big endian"""

        return False

    @property
    def sample_format_endian(self) -> str:
        """Sample format endian"""

        return 'BE' if self.is_sample_format_big_endian else 'LE'

    @property
    def sample_format(self) -> str:
        """Sample format"""

        sample_format: str = f'{self.sample_format_signed}{self.bits_per_sample}'

        if self.bits_per_sample > 8:
            sample_format += f'_{self.sample_format_endian}'

        return sample_format

    ######################################

    @property
    def min_sample_value(self) -> int:
        """Min sample value"""

        return 0

    @property
    def max_sample_value(self) -> int:
        """Max sample value"""

        return pow(2, self.bits_per_sample) - 1

    ############################################################################

    # Offset: ---, Size: 2, Endian: little
    # if PCM, then doesn't exist
    # ExtraParamSize

    # Offset: ---, Size: X, Endian: little
    # space for extra parameters
    # ExtraParams

    ############################################################################
    # The 'data' subchunk contains the size of the data and the actual sound: ##

    # Subchunk2ID ##############################################################

    # Offset: 36, Size: 4, Endian: big
    # Contains the letters 'data'
    # (0x64617461 big-endian form)

    @property
    def subchunk_2_id(self) -> str:
        """Subchunk 2 ID"""

        return 'data'

    @property
    def subchunk_2_id_bytes(self) -> bytes:
        """Subchunk 2 ID bytes"""

        return bytes(self.subchunk_2_id, encoding='ASCII')

    # Subchunk2Size ############################################################

    # Offset: 40, Size: 4, Endian: little
    # == NumSamples * NumChannels * BitsPerSample/8
    # This is the number of bytes in the data.
    # You can also think of this as the size
    # of the read of the subchunk following this number.

    @property
    def subchunk_2_size(self) -> int:
        """Subchunk 2 size"""

        return len(self.data)

    @property
    def subchunk_2_size_bytes(self) -> bytes:
        """Subchunk 2 size bytes"""

        return self.subchunk_2_size.to_bytes(4, byteorder='little', signed=False)

    # Data #####################################################################

    # Offset: 44, Size: *, Endian: little
    # The actual sound data.

    @property
    def data(self) -> bytearray:
        """Data"""

        return self._data

    def add_sample(self, channel_values: list[int], mono_to_all: bool = False) -> None:
        """Add sample"""

        print('Adding sample ', str(channel_values))

        if len(channel_values) > self.num_channels:
            raise ValueError('More values in current sample than channels available')

        if len(channel_values) < self.num_channels:
            if len(channel_values) == 1 and mono_to_all:
                channel_values.extend([channel_values[0]] * (self.num_channels - 1))
            else:
                raise ValueError('Less values in current sample than channels available')

        bytes_per_sample = int(self.bits_per_sample / 8)

        for item in channel_values:

            if item < self.min_sample_value:
                raise ValueError('A value inside the sample is too low')

            if item > self.max_sample_value:
                raise ValueError('A value inside the sample is too high')

            sample_part_bytes: bytes = item.to_bytes(bytes_per_sample, byteorder='little', signed=False)
            self._data.extend(bytearray(sample_part_bytes))

    # OTHER ####################################################################

    def print_format_info(self) -> None:
        """Print format info"""

        print('Number of channels:  ', str(self.num_channels))
        print('Sample Rate:         ', str(self.sample_rate) + ' Hz')
        print('Bits per Sample:     ', str(self.bits_per_sample))
        print('Sample Format:       ', str(self.sample_format))

    def write_to_file(self, arg_file_name='output.wav') -> None:
        """Write to file"""

        print('')
        print('WRITING AUDIO DATA TO FILE')

        self.print_format_info()

        print('File name:           ', arg_file_name)

        with open(arg_file_name, 'wb') as fh_wav:

            print('Writing RIFF header ...', end='')

            fh_wav.write(self.chunk_id_bytes)
            fh_wav.write(self.chunk_size_bytes)
            fh_wav.write(self.format_bytes)

            print(' done.')

            print('Writing "fmt " subchunk ...', end='')

            fh_wav.write(self.subchunk_1_id_bytes)
            fh_wav.write(self.subchunk_1_size_bytes)
            fh_wav.write(self.audio_format_bytes)
            fh_wav.write(self.num_channels_bytes)
            fh_wav.write(self.sample_rate_bytes)
            fh_wav.write(self.byte_rate_bytes)
            fh_wav.write(self.block_align_bytes)
            fh_wav.write(self.bits_per_sample_bytes)

            print(' done.')

            print('Writing "data" subchunk.')

            fh_wav.write(self.subchunk_2_id_bytes)
            fh_wav.write(self.subchunk_2_size_bytes)
            for byte in tqdm(self._data):
                fh_wav.write(bytes([byte]))

        print('Finished writing file.')

    # END ######################################################################



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Tree exec
"""

# imports: library
import argparse
import os.path
import subprocess


DEBUG_INDENT: bool = True
DEBUG_PARENT_LAST: bool = True


class IndentItem:
    """IdentItem"""

    BeforeLast: str = '│   '
    AfterLast: str = '    '
    DirRoot: str = ''
    DirMiddle: str = '├── '
    DirLast: str = '└── '


def quoted(content) -> str:
    """Quoted"""

    return f'"{content}"'


def prefixed(content: str, indents: list[str] = None, tree: str = None) -> str:
    """Prefixed"""

    if indents is None:
        indents: list = []

    prefix: str = ''.join(indents[:-1])

    if tree is None:
        prefix += '-> '
    else:
        prefix += tree

    return prefix + content


def recursive_list(dir_name: str,
                   command: str = None,
                   indents: list[str] = None,
                   tree: str = None
                   ) -> None:
    """Recursive list"""

    if indents is None:
        indents: list = []

    if tree == IndentItem.DirRoot:
        display_name: str = dir_name
    else:
        display_name: str = os.path.basename(dir_name)

    print(prefixed(display_name, indents=indents, tree=tree))

    if command is not None:
        indents_comment: list[str] = indents + [IndentItem.AfterLast]
        indents_subcomment: list[str] = indents_comment + [IndentItem.AfterLast]

        try:
            dir_space_escaped: str = dir_name.replace(' ', '\\' + ' ')
            command_substituted: str = command.replace('$', dir_space_escaped)
            print(prefixed('applying ', indents=indents_comment) + quoted(command_substituted))
            sub_output = subprocess.check_output(command_substituted, shell=True)
        except subprocess.CalledProcessError as err:
            print(prefixed('Error code: ', indents=indents_comment) + quoted(err.returncode))
            print(prefixed('Error message: ', indents=indents_comment) + quoted(err.output))
        else:
            print(prefixed('Return code: ', indents=indents_comment) + quoted('0'))
            print(prefixed('Output: ', indents=indents_comment))
            output_items: list = sub_output.split(b'\n')
            if output_items[-1] == '':
                output_items: list = output_items[:-1]
            for item in output_items:
                item = str(item, encoding='UTF-8')
                print(prefixed(quoted(item), indents=indents_subcomment))

    try:
        dir_list: list[str] = [
            os.path.join(dir_name, item)
            for item in os.listdir(dir_name)
        ]
    except OSError:
        return

    dir_list_filtered: list[str] = [
        os.path.normpath(item)
        for item in dir_list
        if os.path.isdir(item)
    ]

    indents_before: list[str] = indents + [IndentItem.BeforeLast]
    for item in dir_list_filtered[:-1]:
        recursive_list(item, command,
                       indents=indents_before, tree=IndentItem.DirMiddle)

    if len(dir_list_filtered) > 0:
        indents_after: list[str] = indents + [IndentItem.AfterLast]
        recursive_list(dir_list_filtered[-1], command,
                       indents=indents_after, tree=IndentItem.DirLast)


def arg_type_dir_path(path: str) -> str:
    """Arg type: dir path"""

    if not os.path.isdir(path):
        raise argparse.ArgumentTypeError(f'Given argument "{path}" is not a valid directory path')

    return path


def main() -> None:
    """Main"""

    parser = argparse.ArgumentParser(description='Directory list with applied command.')

    parser.add_argument('--root', dest='root', default=os.getcwd(), type=arg_type_dir_path,
                        help='Root directory to recursively execute script from.')

    parser.add_argument('--command', dest='command', default=None,
                        help='Shell command to execute on every directory.')

    args = parser.parse_args()
    root: str = os.path.abspath(str(args.root, encoding='UTF-8'))

    recursive_list(root, command=args.command, tree=IndentItem.DirRoot)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Version and general info
"""

PROGRAM_NAME: str = 'libmonty'

__version__: str = '0.1.1'



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Terminal
"""

# imports: library
import os


def get_cols() -> int:
    """Get columns"""

    try:
        size = os.get_terminal_size()
        cols: int = size.columns
        # lines: int = size.lines
    except OSError:
        cols = 80
        # lines = 20

    return cols



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""OAB processing
"""

# imports: library
from argparse import ArgumentParser, Namespace
import codecs
import re
from typing import Generator

# where to get the input file:
# C:\Users\%username%\AppData\Local\Microsoft\Outlook
# -> Offline Address Books

DEFAULT_INPUT_FILE_PATH: str = 'aa-private-data/udetails-adjusted.oab'
DEFAULT_OUTPUT_UNITS_FILE_PATH: str = 'zz-private-output/contact-list-units.txt'
DEFAULT_OUTPUT_UNITS_SPLIT_FILE_PATH: str = 'zz-private-output/contact-list-units-split.txt'
DEFAILT_OUTPUT_ELEMENTS_FILE_PATH: str = 'zz-private-output/contact-list-elements.csv'
SRC_DELIMITER: str = '/o='


# functions

def file_len(filename: str) -> int:
    """File length"""

    with codecs.open(filename, 'r', encoding='UTF-8', errors='replace') as file:
        for index, _ in enumerate(file):
            pass
    return index + 1


def generator_by_line(filename: str) -> Generator:
    """Generator by line"""

    with open(filename, 'w', encoding='UTF-8') as fh_out:

        while True:
            row: str or bool = (yield True)

            if row is None:
                continue

            if row is False:
                fh_out.close()
                break

            fh_out.write(row)


def next_by_line(generator: Generator, content: str or None) -> None:
    """Next by line"""

    generator.send(content)


def end_by_line(generator: Generator) -> None:
    """End by line"""

    try:
        generator.send(False)
    except StopIteration:
        pass


def is_identifier(item: str) -> bool:
    """Is identifier?"""

    item_list: list = item.split("-")

    if len(item_list) >= 5 and len(item_list[0]) == 8:
        return True

    return False


def elements_line_compose(item_count: int,
                          item_elements: dict,
                          id_label_org: str,
                          id_label_tech: str) -> str:
    """Elements line compose"""

    composed: str = str(item_count)

    composed += f';{item_elements[id_label_tech]}'
    composed += f';{item_elements["full_name"]}'
    try:
        composed += f';{item_elements[id_label_org]}'
    except KeyError:
        composed += ";"
    composed += f';{item_elements["first_name"]}'
    composed += f';{item_elements["last_name"]}'

    composed += '\n'

    return composed


def output_elements_line(content_item_count: int,
                         content_item_elements: dict,
                         gen_file_elements: Generator,
                         id_label_org: str,
                         id_label_tech: str) -> int:
    """Output elements line"""

    # add to items count and print status
    content_item_count += 1
    print(f'Item written. (No. {content_item_count})')

    output: str = elements_line_compose(content_item_count,
                                        content_item_elements,
                                        id_label_org,
                                        id_label_tech)
    next_by_line(gen_file_elements, output)

    return content_item_count


def process(org, id_label_org: str, id_label_tech: str) -> None:
    """Process"""

    # length of data file

    data_file_line_count: int = file_len(DEFAULT_INPUT_FILE_PATH)
    print(f"{data_file_line_count} lines in data file.")

    # variables

    is_end_of_file: bool = False
    line_count: int = 0

    is_content_unit_found: bool = False
    content_unit_count: int = 0

    is_unit_type_org_added: bool = False

    content_item_count: int = 0
    content_item_elements: dict = {}

    search_text: str = ''

    # create elements file generator
    gen_file_elements: Generator = generator_by_line(DEFAILT_OUTPUT_ELEMENTS_FILE_PATH)
    # start the generator
    next_by_line(gen_file_elements, None)

    # loop

    with codecs.open(DEFAULT_INPUT_FILE_PATH, 'r', encoding='UTF-8', errors='replace') as file_in, \
         open(DEFAULT_OUTPUT_UNITS_FILE_PATH, "w", encoding='UTF-8') as file_out_units, \
         open(DEFAULT_OUTPUT_UNITS_SPLIT_FILE_PATH, "w", encoding='UTF-8') as file_out_units_split:

        while True:

            if is_end_of_file:
                break

            # if next unit not yet found: get next line from file
            if not is_content_unit_found:

                line = file_in.readline()

                # if line is empty end of file is reached
                if not line:
                    is_end_of_file = True
                else:
                    line_count += 1
                    print(f'Processing line {line_count} of {data_file_line_count}')

                    search_text += line

            # search for the beginning of the unit

            i_search_1 = search_text.find(SRC_DELIMITER)

            # if unit beginning not found: skip to processing next line
            if i_search_1 == -1:
                is_content_unit_found = False
                continue

            # search for the beginning of the next unit (~ end of the current one)

            i_search_2 = search_text.find(SRC_DELIMITER, i_search_1 + 1)

            if i_search_2 == -1 and not is_end_of_file:
                is_content_unit_found = False
                continue

            # else
            is_content_unit_found = True

            # add to units count and print status
            content_unit_count += 1
            print(f'Content unit identified. (No. {content_unit_count})')

            # isolate current content unit
            content_unit_text: str = search_text[i_search_1:i_search_2 - 1]

            # write to 'units' file
            file_out_units.write(f'[{content_unit_count:>6}] \'{content_unit_text}\'\n')

            # remove current content unit from search string
            search_text: str = search_text[i_search_2:len(search_text)-1]

            # split content unit string by NUL separators
            content_unit_list: list = content_unit_text.split('\u0000')

            # write to 'units_split' file
            line: str = ''
            for index, unit_item_text in enumerate(content_unit_list):
                if line == '':
                    line += f'[#{content_unit_count:>6}] \'{unit_item_text}\'\n'
                else:
                    line += f'        [{index:>2}] \'{unit_item_text}\'\n'
            file_out_units_split.write(line)

            if f'/o={org}/' in content_unit_list[0]:

                # if next ORG line reached but an ORG line is already added
                # but elements not yet output because ID-ORG missing: output to elements
                if is_unit_type_org_added:
                    content_item_count = output_elements_line(
                        content_item_count,
                        content_item_elements,
                        gen_file_elements,
                        id_label_org,
                        id_label_tech)
                    is_unit_type_org_added = False

                # szervezeti bejegyzés átugrása
                if content_unit_list[1].find('org_') == 0:
                    continue

                # túl rövid, egyéb típusú bejegyzés átugrása
                if len(content_unit_list) < 5:
                    line = f'Nincs elég elem az {org}-s sorban! ({line_count}.)\n'
                    line += f'{content_unit_list}'
                    print(line)
                    continue

                # empty content elements list
                content_item_elements = {}

                # ha szám van a i4 elemben, akkor a vezetéknév és keresztnév nincs külön

                # old check
                # if (re.findall("[0-9]", content_unit_list[4]) and
                #         not re.findall("[(][A-Z0-9]{6}[)]", content_unit_list[4])):

                if is_identifier(content_unit_list[4]):

                    id_tech: str = content_unit_list[2]
                    full_name: str = content_unit_list[3]
                    first_name: str = ''
                    last_name: str = ''

                else:
                    first_name: str = content_unit_list[1]
                    last_name: str = content_unit_list[2]
                    id_tech: str = content_unit_list[3]
                    full_name: str = content_unit_list[4]

                    if re.findall('[(][A-Z0-9]{6}[)]', full_name):
                        full_name: str = "".join(re.split('[(][A-Z0-9]{6}[)]', full_name))
                        full_name: str = full_name.strip()

                # hozzáadás az elemlistához
                content_item_elements[id_label_tech] = id_tech
                content_item_elements['full_name'] = full_name
                content_item_elements['first_name'] = first_name
                content_item_elements['last_name'] = last_name

                is_unit_type_org_added = True

            # check for ID-ORG in all content units

            id_org: str = ''

            for unit_item_text in content_unit_list:

                if len(unit_item_text) == 6 and re.findall('[A-Z0-9]{6}', unit_item_text):
                    id_org: str = unit_item_text

            if len(id_org) != 0:
                content_item_elements[id_label_org] = id_org

            # if ID-ORG found and ORG line already added: output to elements

            if is_unit_type_org_added and len(id_org) != 0:
                content_item_count = output_elements_line(
                    content_item_count,
                    content_item_elements,
                    gen_file_elements,
                    id_label_org,
                    id_label_tech)
                is_unit_type_org_added = False

    # end elements generator
    end_by_line(gen_file_elements)


def main() -> None:
    """Main"""

    parser = ArgumentParser(prog='OAB processing')

    parser.add_argument('-o', '--org',
                        help='Organization',
                        required=True,
                        dest='org')

    parser.add_argument('-i', '--id-label-org',
                        help='ID label (organization)',
                        required=True,
                        dest='id_label_org')

    parser.add_argument('-t', '--id-label-tech',
                        help='ID label (tech)',
                        required=True,
                        dest='id_label_tech')

    args: Namespace = parser.parse_args()

    process(org=args.org,
            id_label_org=args.id_label_org,
            id_label_tech=args.id_label_tech)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Output decorators with ASCII escape sequences
   Version 1.1

https://en.wikipedia.org/wiki/ANSI_escape_code
"""

# imports: library
from typing import Callable


def bold_text(function: Callable) -> Callable:
    """Bold text"""

    def new_function(printable: str) -> str:
        return f'\x1b[1m{function(printable)}\x1b[0m'

    return new_function


def faint_text(function: Callable) -> Callable:
    """Faint text"""

    def new_function(printable: str) -> str:
        return f'\x1b[2m{function(printable)}\x1b[0m'

    return new_function


def italic_text(function: Callable) -> Callable:
    """Italic text"""

    def new_function(printable: str) -> str:
        return f'\x1b[3m{function(printable)}\x1b[0m'

    return new_function


def underlined_text(function: Callable) -> Callable:
    """Underlined text"""

    def new_function(printable: str) -> str:
        return f'\x1b[4m{function(printable)}\x1b[0m'

    return new_function


def blinking_text(function: Callable) -> Callable:
    """Blinking text"""

    def new_function(printable: str) -> str:
        return f'\x1b[5m{function(printable)}\x1b[0m'

    return new_function


def image_negative_text(function: Callable) -> Callable:
    """Image negative text"""

    def new_function(printable: str) -> str:
        return f'\x1b[7m{function(printable)}\x1b[0m'

    return new_function


def primary_font_text(function: Callable) -> Callable:
    """Primary font text"""

    def new_function(printable: str) -> str:
        return f'\x1b[10m{function(printable)}\x1b[0m'

    return new_function


def alternate_font_text(function: Callable) -> Callable:
    """Alternate font text"""

    def new_function(printable: str, arg_alternate_font_no) -> str:
        inserted_no = '1'
        if int(arg_alternate_font_no) in range(1, 10):  # 0 < x < 10 #
            inserted_no = str(arg_alternate_font_no)
        return f'\x1b[1{inserted_no}m{function(printable, arg_alternate_font_no)}\x1b[0m'

    return new_function


def black_text(function: Callable) -> Callable:
    """Black text"""

    def new_function(printable: str) -> str:
        return f'\x1b[30m{function(printable)}\x1b[0m'

    return new_function


def red_text(function: Callable) -> Callable:
    """Red text"""

    def new_function(printable: str) -> str:
        return f'\x1b[31m{function(printable)}\x1b[0m'

    return new_function


def green_text(function: Callable) -> Callable:
    """Green text"""

    def new_function(printable: str) -> str:
        return f'\x1b[32m{function(printable)}\x1b[0m'

    return new_function


def yellow_text(function: Callable) -> Callable:
    """Yellow text"""

    def new_function(printable: str) -> str:
        return f'\x1b[33m{function(printable)}\x1b[0m'

    return new_function


def blue_text(function: Callable) -> Callable:
    """Blue text"""

    def new_function(printable: str) -> str:
        return f'\x1b[34m{function(printable)}\x1b[0m'

    return new_function


def magenta_text(function: Callable) -> Callable:
    """Magenta text"""

    def new_function(printable: str) -> str:
        return f'\x1b[35m{function(printable)}\x1b[0m'

    return new_function


def cyan_text(function: Callable) -> Callable:
    """Cyan text"""

    def new_function(printable: str) -> str:
        return f'\x1b[36m{function(printable)}\x1b[0m'

    return new_function


def white_text(function: Callable) -> Callable:
    """White text"""

    def new_function(printable: str) -> str:
        return f'\x1b[37m{function(printable)}\x1b[0m'

    return new_function


def bg_black_text(function: Callable) -> Callable:
    """Background black text"""

    def new_function(printable: str) -> str:
        return f'\x1b[40m{function(printable)}\x1b[0m'

    return new_function


def bg_red_text(function: Callable) -> Callable:
    """Background red text"""

    def new_function(printable: str) -> str:
        return f'\x1b[41m{function(printable)}\x1b[0m'

    return new_function


def bg_green_text(function: Callable) -> Callable:
    """Background green text"""

    def new_function(printable: str) -> str:
        return f'\x1b[42m{function(printable)}\x1b[0m'

    return new_function


def bg_yellow_text(function: Callable) -> Callable:
    """Background yellow text"""

    def new_function(printable: str) -> str:
        return f'\x1b[43m{function(printable)}\x1b[0m'

    return new_function


def bg_blue_text(function: Callable) -> Callable:
    """Background blue text"""

    def new_function(printable: str) -> str:
        return f'\x1b[44m{function(printable)}\x1b[0m'

    return new_function


def bg_magenta_text(function: Callable) -> Callable:
    """Background magenta text"""

    def new_function(printable: str) -> str:
        return f'\x1b[45m{function(printable)}\x1b[0m'

    return new_function


def bg_cyan_text(function: Callable) -> Callable:
    """Background cyan text"""

    def new_function(printable: str) -> str:
        return f'\x1b[46m{function(printable)}\x1b[0m'

    return new_function


def bg_white_text(function: Callable) -> Callable:
    """Background white text"""

    def new_function(printable: str) -> str:
        return f'\x1b[47m{function(printable)}\x1b[0m'

    return new_function


def framed_text(function: Callable) -> Callable:
    """Framed text"""

    def new_function(printable: str) -> str:
        return f'\x1b[51m{function(printable)}\x1b[0m'

    return new_function


def encircled_text(function: Callable) -> Callable:
    """Encircled text"""

    def new_function(printable: str) -> str:
        return f'\x1b[52m{function(printable)}\x1b[0m'

    return new_function


def overlined_text(function: Callable) -> Callable:
    """Overlined text"""

    def new_function(printable: str) -> str:
        return f'\x1b[53m{function(printable)}\x1b[0m'

    return new_function



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Colors
"""

COLORS: dict[str, str] = {
    'F0F8FF': 'AliceBlue',
    'FAEBD7': 'AntiqueWhite',
    '00FFFF': 'Aqua / Cyan',
    '7FFFD4': 'Aquamarine',
    'F0FFFF': 'Azure',
    'F5F5DC': 'Beige',
    'FFE4C4': 'Bisque',
    '000000': 'Black',
    'FFEBCD': 'BlanchedAlmond',
    '0000FF': 'Blue',
    '8A2BE2': 'BlueViolet',
    'A52A2A': 'Brown',
    'DEB887': 'BurlyWood',
    '5F9EA0': 'CadetBlue',
    '7FFF00': 'Chartreuse',
    'D2691E': 'Chocolate',
    'FF7F50': 'Coral',
    '6495ED': 'CornflowerBlue',
    'FFF8DC': 'Cornsilk',
    'DC143C': 'Crimson',
    '00008B': 'DarkBlue',
    '008B8B': 'DarkCyan',
    'B8860B': 'DarkGoldenRod',
    'A9A9A9': 'DarkGray / DarkGrey',
    '006400': 'DarkGreen',
    'BDB76B': 'DarkKhaki',
    '8B008B': 'DarkMagenta',
    '556B2F': 'DarkOliveGreen',
    'FF8C00': 'DarkOrange',
    '9932CC': 'DarkOrchid',
    '8B0000': 'DarkRed',
    'E9967A': 'DarkSalmon',
    '8FBC8F': 'DarkSeaGreen',
    '483D8B': 'DarkSlateBlue',
    '2F4F4F': 'DarkSlateGray / DarkSlateGrey',
    '00CED1': 'DarkTurquoise',
    '9400D3': 'DarkViolet',
    'FF1493': 'DeepPink',
    '00BFFF': 'DeepSkyBlue',
    '696969': 'DimGray / DimGrey',
    '1E90FF': 'DodgerBlue',
    'B22222': 'FireBrick',
    'FFFAF0': 'FloralWhite',
    '228B22': 'ForestGreen',
    'FF00FF': 'Fuchsia / Magenta',
    'DCDCDC': 'Gainsboro',
    'F8F8FF': 'GhostWhite',
    'FFD700': 'Gold',
    'DAA520': 'GoldenRod',
    '808080': 'Gray / Grey',
    '008000': 'Green',
    'ADFF2F': 'GreenYellow',
    'F0FFF0': 'HoneyDew',
    'FF69B4': 'HotPink',
    'CD5C5C': 'IndianRed',
    '4B0082': 'Indigo',
    'FFFFF0': 'Ivory',
    'F0E68C': 'Khaki',
    'E6E6FA': 'Lavender',
    'FFF0F5': 'LavenderBlush',
    '7CFC00': 'LawnGreen',
    'FFFACD': 'LemonChiffon',
    'ADD8E6': 'LightBlue',
    'F08080': 'LightCoral',
    'E0FFFF': 'LightCyan',
    'FAFAD2': 'LightGoldenRodYellow',
    'D3D3D3': 'LightGray / LightGrey',
    '90EE90': 'LightGreen',
    'FFB6C1': 'LightPink',
    'FFA07A': 'LightSalmon',
    '20B2AA': 'LightSeaGreen',
    '87CEFA': 'LightSkyBlue',
    '778899': 'LightSlateGray / LightSlateGrey',
    'B0C4DE': 'LightSteelBlue',
    'FFFFE0': 'LightYellow',
    '00FF00': 'Lime',
    '32CD32': 'LimeGreen',
    'FAF0E6': 'Linen',
    '800000': 'Maroon',
    '66CDAA': 'MediumAquaMarine',
    '0000CD': 'MediumBlue',
    'BA55D3': 'MediumOrchid',
    '9370DB': 'MediumPurple',
    '3CB371': 'MediumSeaGreen',
    '7B68EE': 'MediumSlateBlue',
    '00FA9A': 'MediumSpringGreen',
    '48D1CC': 'MediumTurquoise',
    'C71585': 'MediumVioletRed',
    '191970': 'MidnightBlue',
    'F5FFFA': 'MintCream',
    'FFE4E1': 'MistyRose',
    'FFE4B5': 'Moccasin',
    'FFDEAD': 'NavajoWhite',
    '000080': 'Navy',
    'FDF5E6': 'OldLace',
    '808000': 'Olive',
    '6B8E23': 'OliveDrab',
    'FFA500': 'Orange',
    'FF4500': 'OrangeRed',
    'DA70D6': 'Orchid',
    'EEE8AA': 'PaleGoldenRod',
    '98FB98': 'PaleGreen',
    'AFEEEE': 'PaleTurquoise',
    'DB7093': 'PaleVioletRed',
    'FFEFD5': 'PapayaWhip',
    'FFDAB9': 'PeachPuff',
    'CD853F': 'Peru',
    'FFC0CB': 'Pink',
    'DDA0DD': 'Plum',
    'B0E0E6': 'PowderBlue',
    '800080': 'Purple',
    '663399': 'RebeccaPurple',
    'FF0000': 'Red',
    'BC8F8F': 'RosyBrown',
    '4169E1': 'RoyalBlue',
    '8B4513': 'SaddleBrown',
    'FA8072': 'Salmon',
    'F4A460': 'SandyBrown',
    '2E8B57': 'SeaGreen',
    'FFF5EE': 'SeaShell',
    'A0522D': 'Sienna',
    'C0C0C0': 'Silver',
    '87CEEB': 'SkyBlue',
    '6A5ACD': 'SlateBlue',
    '708090': 'SlateGray / SlateGrey',
    'FFFAFA': 'Snow',
    '00FF7F': 'SpringGreen',
    '4682B4': 'SteelBlue',
    'D2B48C': 'Tan',
    '008080': 'Teal',
    'D8BFD8': 'Thistle',
    'FF6347': 'Tomato',
    '40E0D0': 'Turquoise',
    'EE82EE': 'Violet',
    'F5DEB3': 'Wheat',
    'FFFFFF': 'White',
    'F5F5F5': 'WhiteSmoke',
    'FFFF00': 'Yellow',
    '9ACD32': 'YellowGreen'
}



#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Email check
"""

# imports: library
from argparse import ArgumentParser, Namespace
import os.path

# imports: requirements
from email_validator import validate_email, EmailSyntaxError, EmailUndeliverableError

# imports: project
from libmonty.environment.arguments import type_file_path


def main():
    """Main"""

    parser = ArgumentParser()

    parser.add_argument('-i', '--input-file', metavar='PATH',
                        required=True, type=type_file_path,
                        dest='input_file')

    parser.add_argument('-o', '--output-file', metavar='PATH',
                        default=None, type=type_file_path,
                        dest='output_file')

    args: Namespace = parser.parse_args()

    print('Input file: ', args.input_file)
    with open(args.input_file, 'r', encoding='UTF-8') as fh_source:
        email_list: list[str] = fh_source.readlines()

    email_list: list[str] = list(map(lambda x: x.strip(), email_list))

    if args.output_file is None:
        input_file_part_name, input_file_part_ext = os.path.splitext(args.input_file)
        output_file = f'{input_file_part_name}-output{input_file_part_ext}'
    else:
        output_file = args.output_file

    with open(output_file, 'w', encoding='UTF-8') as fh_result:

        for email in email_list:

            print('Checking:', email)

            try:
                # Validate.
                valid = validate_email(email)

                # Update with the normalized form.
                print('->  Valid:', valid.email)
                fh_result.write(f'{email:<31} => {valid.email}\n')

            except EmailSyntaxError as exc:
                # email is not valid, exception message is human-readable
                print(str(exc))
                fh_result.write(f'Invalid syntax: {email}')

            except EmailUndeliverableError as exc:
                print(str(exc))
                fh_result.write(f'Undeliverable: {email}')


if __name__ == '__main__':
    main()
